import React, { Component } from 'react';
import { createStore } from 'redux';
import { Provider, connect } from 'react-redux';

var deepCopy = function deepCopy(ele) {
  if (typeof ele === "object" && ele !== null) {
    if (Array.isArray(ele)) {
      return [].concat(ele.map(function (e) {
        return deepCopy(e);
      }));
    }

    var result = {};
    Object.keys(ele).forEach(function (e) {
      result[e] = deepCopy(ele[e]);
    });
    return result;
  }

  return ele;
};

var jsonToQueryString = function jsonToQueryString(json) {
  return '?' + Object.keys(json).map(function (key) {
    if (json[key] !== null) {
      return encodeURIComponent(key) + '=' + encodeURIComponent(json[key]);
    }

    return '';
  }).join('&');
};

var queryStringToJSON = function queryStringToJSON(queryString) {
  if (queryString.indexOf('?') > -1) {
    queryString = queryString.split('?')[1];
  }

  var pairs = queryString.split('&');
  var result = {};
  pairs.forEach(function (pair) {
    pair = pair.split('=');
    result[pair[0]] = decodeURIComponent(pair[1] || '');
  });
  return result;
};

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

var Utils = {
  __proto__: null,
  deepCopy: deepCopy,
  jsonToQueryString: jsonToQueryString,
  queryStringToJSON: queryStringToJSON,
  getDisplayName: getDisplayName
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var SET_STATE = "SET_STATE";
var RESET_STATE = "RESET_STATE";

var generateStore = function generateStore(reactNCState) {
  return {
    reactNCState: reactNCState
  };
};

var reduxReducer = (function (store, action) {
  if (store === void 0) {
    store = generateStore(deepCopy(CTRL.initialState));
  }

  switch (action.key) {
    case SET_STATE:
      if (action.callback) {
        console.warn('React NC Redux does not support callback function in setState()');
      }

      var currentState = store.reactNCState;
      var updatingState = currentState;

      if (typeof action.payload === 'function') {
        updatingState = action.payload(currentState);
      } else {
        updatingState = action.payload;
      }

      return generateStore(_extends({}, currentState, updatingState));

    case RESET_STATE:
      return generateStore(deepCopy(CTRL.initialState));

    default:
      return store;
  }
});
var setState = function setState(type, state, callback) {
  if (type === void 0) {
    type = "UNKNOWN";
  }

  if (state === void 0) {
    state = {};
  }

  return {
    key: SET_STATE,
    type: type,
    payload: state,
    callback: callback
  };
};
var resetState = function resetState() {
  return {
    key: RESET_STATE,
    type: RESET_STATE
  };
};

var CTRL = {
  app: undefined,
  config: {},
  actions: {},
  debug: false,
  initialState: {},
  log: function log() {
    var _console;

    for (var _len = arguments.length, params = new Array(_len), _key = 0; _key < _len; _key++) {
      params[_key] = arguments[_key];
    }

    if (CTRL.debug) (_console = console).log.apply(_console, ["React-nc: "].concat(params));
  },
  setConfig: function setConfig(config) {
    if (config === void 0) {
      config = {};
    }

    CTRL.config = config;
  },
  forceUpdate: function forceUpdate() {
    if (!CTRL.app) {
      throw new Error("Application wrapped by StateControl is not found");
    }

    CTRL.app.forceUpdate();
  },
  createAction: function createAction(actionName, action) {
    if (typeof actionName !== "string") {
      throw new Error("Invalid 'actionName' for createAction");
    } else if (!actionName) {
      throw new Error("'actionName' cannot be empty for createAction");
    } else if (CTRL.actions[actionName]) {
      throw new Error("Action '" + actionName + "' is already defined");
    }

    var result = function result() {
      if (typeof action === "object") {
        CTRL.setState(actionName)(action);
      } else if (typeof action === "function") {
        CTRL.setState(actionName)(action.apply(void 0, arguments));
      }
    };

    CTRL.actions[actionName] = result;
    return result;
  },
  removeAction: function removeAction(actionName) {
    if (!CTRL.actions[actionName]) {
      throw new Error("Action '" + actionName + "' is not defined");
    }

    delete CTRL.actions[actionName];
  },
  setState: function setState$1(actionType, callback) {
    if (actionType === void 0) {
      actionType = "UNKNOWN";
    }

    var state = undefined;

    if (typeof actionType !== "string") {
      state = actionType;
      actionType = "UNKNOWN";
    }

    var dispatch = function dispatch(state, callback) {
      if (!CTRL.app) {
        throw new Error("Application wrapped by StateControl is not found");
      }

      CTRL.app.store.dispatch(setState(actionType, state, callback));
    };

    if (typeof state !== "undefined") {
      dispatch(state, callback);
      return;
    }

    return dispatch;
  },
  resetState: function resetState$1() {
    if (!CTRL.app) {
      throw new Error("Application wrapped by StateControl is not found");
    }

    CTRL.app.store.dispatch(resetState());
  },
  initializeState: function initializeState(state) {
    CTRL.initialState = state;

    if (CTRL.app) {
      CTRL.resetState();
    }
  },

  get state() {
    if (!CTRL.app) {
      throw new Error("Application wrapped by StateControl is not found");
    }

    return CTRL.app.store.getState().reactNCState;
  }

};

var StateControl = function StateControl(Comp) {
  var StateControlComponent = /*#__PURE__*/function (_Component) {
    _inheritsLoose(StateControlComponent, _Component);

    function StateControlComponent(props) {
      var _this;

      _this = _Component.call(this, props) || this;
      CTRL.app = _assertThisInitialized(_this);
      _this.store = createStore(reduxReducer, window.__REDUX_DEVTOOLS_EXTENSION__ && window.__REDUX_DEVTOOLS_EXTENSION__());
      return _this;
    }

    var _proto = StateControlComponent.prototype;

    _proto.render = function render() {
      var props = this.props;
      return /*#__PURE__*/React.createElement(Provider, {
        store: this.store
      }, /*#__PURE__*/React.createElement(Comp, props));
    };

    return StateControlComponent;
  }(Component);

  StateControlComponent.displayName = "StateControl(" + getDisplayName(Comp) + ")";
  return StateControlComponent;
};

var StateConnect = function StateConnect(mapStateToUpdate, withRef, connectParams) {
  if (mapStateToUpdate === void 0) {
    mapStateToUpdate = function mapStateToUpdate(state) {
      return deepCopy(CTRL.state);
    };
  }

  if (withRef === void 0) {
    withRef = false;
  }

  return function (Comp) {
    var mapStateToUpdateAll = CTRL.config.mapStateToUpdateAll || {};
    var fromAll = typeof mapStateToUpdateAll === 'object' ? function () {
      return mapStateToUpdateAll;
    } : mapStateToUpdateAll;
    var fromMap = typeof mapStateToUpdate === 'object' ? function () {
      return mapStateToUpdate;
    } : mapStateToUpdate;
    var ConnectedComp = connect.apply(void 0, [function (store) {
      for (var _len = arguments.length, params = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        params[_key - 1] = arguments[_key];
      }

      return {
        reactNCStateStringified: JSON.stringify(_extends({}, fromAll.apply(void 0, [store.reactNCState].concat(params)), fromMap.apply(void 0, [store.reactNCState].concat(params))))
      };
    }].concat(Array.isArray(connectParams) ? connectParams : [null, null, {
      withRef: withRef
    }]))(Comp);
    return ConnectedComp;
  };
};

export default CTRL;
export { StateConnect, StateControl, Utils };
//# sourceMappingURL=index.modern.js.map
